# Generated by Django 2.0.5 on 2018-06-01 03:53

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UrlMap',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('full_url', models.CharField(max_length=256)),
                ('short_url', models.CharField(db_index=True, max_length=50, unique=True)),
                ('usage_count', models.IntegerField(default=0)),
                ('max_count', models.IntegerField(default=-1)),
                ('lifespan', models.IntegerField(default=-1)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_expired', models.DateTimeField()),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UrlProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enabled', models.BooleanField(default=True)),
                ('max_urls', models.IntegerField(default=-1)),
                ('max_concurrent_urls', models.IntegerField(default=100)),
                ('default_lifespan', models.IntegerField(default=120)),
                ('default_max_uses', models.IntegerField(default=-1)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.shortcuts import redirect
from django.http import HttpResponse
from shortener import shortener


# Create your views here.
def test(request, link):
    data = shortener.create(request.user, link)
    return HttpResponse(data)


def expand(request, link):
    try:
        link = shortener.expand(link)
        return redirect(link)  # TODO: permanent=True
    except Exception as e:
        return HttpResponse(e.args)



from django.test import TestCase
from django.contrib.auth.models import AnonymousUser, User
from shortener.models import UrlMap, UrlProfile
from shortener import shortener


class UrlMapTestCase(TestCase):
    def setUp(self):
        self.bob = User.objects.create_user('bob', 'bob@bob.com', 'bobpassword')
        self.alice = User.objects.create_user('alice', 'alive@alice.com', 'alicepassword')

    def test_url_creation(self):
        url = shortener.create(self.bob, "http://devget.net/")
        self.assertEqual(shortener.expand(url), "http://devget.net/")

        # TODO: write tests to test security features



from shortener.models import UrlMap, UrlProfile
from django.conf import settings
from django.db import IntegrityError
from datetime import datetime, timedelta
from django.utils import timezone

import random
import string


def get_random(tries=0):
    if hasattr(settings, 'SHORTENER_LENGTH'):
        length = settings.SHORTENER_LENGTH
    else:
        length = 5

    length += tries

    # return ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(length))

    # Removed l, I, 1
    dictionary = "ABCDEFGHJKLMNOPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz234567890"
    return ''.join(random.choice(dictionary) for _ in range(length))


def create(user, link):
    # check if user allowed to save link
    try:
        # use user settings
        p = UrlProfile.objects.get(user=user)
        enabled = p.enabled
        max_urls = p.max_urls
        max_concurrent = p.max_concurrent_urls
        lifespan = p.default_lifespan
        max_uses = p.default_max_uses
    except UrlProfile.DoesNotExist:
        # Use defaults from settings
        if hasattr(settings, 'SHORTENER_ENABLED'):
            enabled = settings.SHORTENER_ENABLED
        else:
            enabled = True

        if hasattr(settings, 'SHORTENER_MAX_URLS'):
            max_urls = settings.SHORTENER_MAX_URLS
        else:
            max_urls = -1

        if hasattr(settings, 'SHORTENER_MAX_CONCURRENT'):
            max_concurrent = settings.SHORTENER_MAX_CONCURRENT
        else:
            max_concurrent = -1

        if hasattr(settings, 'SHORTENER_LIFESPAN'):
            lifespan = settings.SHORTENER_LIFESPAN
        else:
            lifespan = -1

        if hasattr(settings, 'SHORTENER_MAX_USES'):
            max_uses = settings.SHORTENER_MAX_USES
        else:
            max_uses = -1

    # Ensure User is allowed to create
    if not enabled:
        raise PermissionError("User not allowed to access create")

    # Expiry date, -1 to disable
    if lifespan != -1:
        expiry_date = datetime.now() + timedelta(seconds=lifespan)
    else:
        expiry_date = datetime.max

    # Ensure user has not met max_urls quota
    if max_urls != -1:
        if UrlMap.objects.filter(user=user).count() >= max_urls:
            raise PermissionError("User has met url quota")

    # Ensure user has not met concurrent urls quota
    if max_concurrent != -1:
        if UrlMap.objects.filter(user=user, date_expired__gt=datetime.now()).count() >= max_concurrent:
            raise PermissionError("User has met concurrent quota")

    # Try up to three times to generate a random number without duplicates.
    # Each time increase the number of allowed characters
    for tries in range(3):
        try:
            short = get_random(tries)
            m = UrlMap(user=user, full_url=link, short_url=short, max_count=max_uses, date_expired=expiry_date)
            m.save()
            return m.short_url
        except IntegrityError:
            continue

    raise KeyError("Could not generate unique shortlink")


def expand(link):
    try:
        url = UrlMap.objects.get(short_url__exact=link)
    except UrlMap.DoesNotExist:
        raise KeyError("ShortLink Not found")

    # ensure we are within usage counts
    if url.max_count != -1:
        if url.max_count > url.usage_count:
            raise PermissionError("Max usages reached")

    # ensure we are within allowed datetime
    print(datetime.now())
    print(url.date_expired)
    if timezone.now() > url.date_expired:
        raise PermissionError("Link Expired")

    url.usage_count += 1
    url.save()
    return url.full_url




from django.db import models
from django.contrib.auth.models import User
from datetime import datetime, timedelta


# Create your models here.
class UrlMap(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    full_url = models.CharField(max_length=256)
    short_url = models.CharField(max_length=50, unique=True, db_index=True)
    usage_count = models.IntegerField(default=0)
    max_count = models.IntegerField(default=-1)
    lifespan = models.IntegerField(default=-1)
    date_created = models.DateTimeField(auto_now_add=True)
    date_expired = models.DateTimeField()

    def __str__(self):
        return self.full_url


class UrlProfile(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    enabled = models.BooleanField(default=True)
    max_urls = models.IntegerField(default=-1)
    max_concurrent_urls = models.IntegerField(default=100)

    # TODO: Lifespan from SETTINGS
    default_lifespan = models.IntegerField(default=120)
    default_max_uses = models.IntegerField(default=-1)
